ITEM = "Item"
XPORB = "XPOrb"
LEASHKNOT = "LeashKnot"
PAINTING = "Painting"
ARROW = "Arrow"
SNOWBALL = "Snowball"
FIREBALL = "Fireball"
SMALLFIREBALL = "SmallFireball"
THROWNENDERPEARL = "ThrownEnderpearl"
EYEOFENDERSIGNAL = "EyeOfEnderSignal"
THROWNPOTION = "ThrownPotion"
THROWNEXPBOTTLE = "ThrownExpBottle"
ITEMFRAME = "ItemFrame"
WITHERSKULL = "WitherSkull"
PRIMEDTNT = "PrimedTnt"
FALLINGSAND = "FallingSand"
FIREWORKSROCKETENTITY = "FireworksRocketEntity"
ARMORSTAND = "ArmorStand"
BOAT = "Boat"
MINECARTRIDEABLE = "MinecartRideable"
MINECARTCHEST = "MinecartChest"
MINECARTFURNACE = "MinecartFurnace"
MINECARTTNT = "MinecartTNT"
MINECARTHOPPER = "MinecartHopper"
MINECARTSPAWNER = "MinecartSpawner"
MINECARTCOMMANDBLOCK = "MinecartCommandBlock"
MOB = "Mob"
MONSTER = "Monster"
CREEPER = "Creeper"
SKELETON = "Skeleton"
SPIDER = "Spider"
GIANT = "Giant"
ZOMBIE = "Zombie"
SLIME = "Slime"
GHAST = "Ghast"
PIGZOMBIE = "PigZombie"
ENDERMAN = "Enderman"
CAVESPIDER = "CaveSpider"
SILVERFISH = "Silverfish"
BLAZE = "Blaze"
LAVASLIME = "LavaSlime"
ENDERDRAGON = "EnderDragon"
WITHERBOSS = "WitherBoss"
BAT = "Bat"
WITCH = "Witch"
ENDERMITE = "Endermite"
GUARDIAN = "Guardian"
PIG = "Pig"
SHEEP = "Sheep"
COW = "Cow"
CHICKEN = "Chicken"
SQUID = "Squid"
WOLF = "Wolf"
MUSHROOMCOW = "MushroomCow"
SNOWMAN = "SnowMan"
OZELOT = "Ozelot"
VILLAGERGOLEM = "VillagerGolem"
HORSE = "EntityHorse"
RABBIT = "Rabbit"
VILLAGER = "Villager"
ENDERCRYSTAL = "EnderCrystal"
PLAYER = "(ThePlayer)"
