from distutils.core import setup
setup(
    name='pycraft_minetest',
    packages=['pycraft_minetest'],
    version='0.5',
    description='Modified, simplified and improved libraries to code in Python via Minetest.',
    author='alenorfo & gmenegoz',
    author_email='ale.norfo@gmail.com',
    url='https://github.com/sprintingkiwi/pycraft_lib',
    download_url='https://github.com/sprintingkiwi/pycraft_lib/tarball/0.5',
    keywords=['game', 'development', 'learning', 'education', 'turtle'],
    classifiers=[],
)
